#include "actor.h"

/*--------------------------------------------------------
    function: makeCube
    purpose:		builds  cube
  -------------------------------------------------------*/
void Actor::makeCube(void)
{
	aspect.Init(8,12);
	aspect.addVertex( -1,  1,  1);
	aspect.addVertex(  1,  1,  1);
	aspect.addVertex(  1, -1,  1);
	aspect.addVertex( -1, -1,  1);

    aspect.addVertex( -1,  1, -1);
	aspect.addVertex(  1,  1, -1);
	aspect.addVertex(  1, -1, -1);
	aspect.addVertex( -1, -1, -1);
	
	aspect.addFace( 0, 1, 2);
	aspect.addFace( 2, 3, 0);
	
	aspect.addFace( 1, 5, 6);
	aspect.addFace( 6, 2, 1);
	
	aspect.addFace( 4, 5, 1);
	aspect.addFace( 1, 0, 4);

	aspect.addFace( 4, 0, 3);
	aspect.addFace( 3, 7, 4);
	
	aspect.addFace( 6, 5, 4);
	aspect.addFace( 4, 7, 6);
	
	aspect.addFace( 2, 6, 7);
	aspect.addFace( 7, 3, 2);
	
	
	aspect.setFaceSide( 0, 0);
	aspect.setFaceSide( 1, 1);
	aspect.setFaceSide( 2, 0);
	aspect.setFaceSide( 3, 1);
	aspect.setFaceSide( 4, 0);
	aspect.setFaceSide( 5, 1);
	aspect.setFaceSide( 6, 0);
	aspect.setFaceSide( 7, 1);
	aspect.setFaceSide( 8, 0);
	aspect.setFaceSide( 9, 1);
	aspect.setFaceSide( 10, 0);
    aspect.setFaceSide( 11, 1);
  	
	aspect.setFaceColor( 0, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 1, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 2, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 3, 0.0, 0.0, 1.0, 1.0);
    aspect.setFaceColor( 4, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 5, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 6, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 7, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 8, 0.0, 0.0, 1.0, 1.0);
    aspect.setFaceColor( 9, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 10, 0.0, 0.0, 1.0, 1.0);
	aspect.setFaceColor( 11, 0.0, 0.0, 1.0, 1.0);
	
	aspect.updatePivot();
	aspect.computeFaceNormals();
	aspect.computeVertexNormals();
}

// ugly but only for testing purpose
/*--------------------------------------------------------
    function: Render
    purpose:		performs a "realtime" object rendering
  -------------------------------------------------------*/
void Actor::Render()
{
    int nb_vertex = aspect.getNbPoints();
    int nb_faces = aspect.getNbFaces();
    int p1, p2, p3 ;
    
    GLfloat disp_coord_x[nb_vertex], disp_coord_y[nb_vertex], disp_coord_z[nb_vertex];
    
    for ( int i = 0; i < nb_vertex ; i++ )
	{	
		disp_coord_x[i] = aspect.getX(i); 
		disp_coord_y[i] = aspect.getY(i);
		disp_coord_z[i] = aspect.getZ(i);
    }
    
	for ( int i = 0; i < nb_faces ; i++ ) {
		
			p1 = aspect.getPointFromFace( i, 0);
			p2 = aspect.getPointFromFace( i, 1);
			p3 = aspect.getPointFromFace( i, 2);
			
			glBegin(GL_TRIANGLES);
			    glNormal3f( aspect.getFaceNormalX(i), aspect.getFaceNormalY(i), aspect.getFaceNormalZ(i) );
			    
			    glVertex3f( disp_coord_x[p1], disp_coord_y[p1], disp_coord_z[p1]);
			    glVertex3f( disp_coord_x[p3], disp_coord_y[p3], disp_coord_z[p3]);
			    glVertex3f( disp_coord_x[p2], disp_coord_y[p2], disp_coord_z[p2]);
			    
			glEnd();
	}
}

/*--------------------------------------------------------
    function: makeList
    purpose:		makes a display list for the object's mesh
  -------------------------------------------------------*/
void Actor::makeList(void)
{
    int nb_vertex = aspect.getNbPoints();
    int nb_faces = aspect.getNbFaces();
    int p1, p2, p3 ;
    
    GLfloat vertex_array[nb_vertex*3];
    GLfloat normal_array[nb_faces*3];
    
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    
    list_id = glGenLists(1);
    
    for (int i = 0; i < nb_vertex ; i++)
	{	
		vertex_array[i*3] =  aspect.getX(i);
		vertex_array[i*3+1] =  aspect.getY(i);
		vertex_array[i*3+2] =  aspect.getZ(i);
    }

    for (int i = 0; i < nb_faces ; i++)
	{	
		normal_array[i*3] =  aspect.getFaceNormalX(i);
		normal_array[i*3 + 1] =  aspect.getFaceNormalY(i);
		normal_array[i*3 + 2] =  aspect.getFaceNormalZ(i);
    }
        
    glVertexPointer(3, GL_FLOAT, 0, vertex_array );
    glNormalPointer(GL_FLOAT, 0, normal_array );
    
    glNewList(list_id, GL_COMPILE);

		  for (int i = 0; i < nb_faces ; i++) 
		  {
		      p1 = aspect.getPointFromFace(i,0);
			  p2 = aspect.getPointFromFace(i,1);
			  p3 = aspect.getPointFromFace(i,2);

			  glBegin(GL_TRIANGLES);
			    glArrayElement(p1);
			    glArrayElement(p2);
			    glArrayElement(p3);
			  glEnd();
		   }
		   
    glEndList();
    
    glDisableClientState(GL_NORMAL_ARRAY);    
    glDisableClientState(GL_VERTEX_ARRAY);
}

/*--------------------------------------------------------
    function: loadFromFile
    purpose:		see Model::Mesh
  -------------------------------------------------------*/
void Actor::loadFromFile(char *filename)
{
    aspect.loadMesh(filename);
}

/*--------------------------------------------------------
    function: scaleMesh
    purpose:		adjusts the size of the mesh
  -------------------------------------------------------*/
void Actor::scaleMesh(float r)
{
    aspect.Scale(r,r,r);
}

/*--------------------------------------------------------
    function: Translate
    purpose:		set the position of the mesh
  -------------------------------------------------------*/
void Actor::Translate(float a, float b, float c)
{
    aspect.Translate(a,b,c);
}
