#include"imageLoader.h"


/*--------------------------------------------------------------------------
                                              getWidth
    PURPOSE: returns image information
    RETURN: the width of the image.
---------------------------------------------------------------------------*/
int imageLoader::getWidth(void)
{
    return(width);
}

/*--------------------------------------------------------------------------
                                              getHeight
    PURPOSE: returns image information
    RETURN: the height of the image.
---------------------------------------------------------------------------*/
int imageLoader::getHeight(void)
{
    return(height);
}

/*--------------------------------------------------------------------------
                                              getSize
    PURPOSE: returns image information
    RETURN: the size of the image IN PIXEL NOT BYTES !
---------------------------------------------------------------------------*/
int imageLoader::getSize(void)
{
    return(size);
}

/*--------------------------------------------------------------------------
                                              resetImage
    PURPOSE: resets all parameters and frees memory
    RETURN: true if there was something to reset else returns false.
---------------------------------------------------------------------------*/
bool imageLoader::resetImage(void)
{
  if (loaded) 
  {
    delete[] rvb_map ;
    loaded = false ;
    width = height = 0;
    has_alpha = false;
    return true;
  }
  return false;
}

/*--------------------------------------------------------------------------
                                              loadImage
    PURPOSE: loads an image according to the given type
    RETURN: true if loading was succesful else returns false.
---------------------------------------------------------------------------*/
bool imageLoader::loadImage(int type, char *filename)
{
  switch( type )
  {
    case GIF:
            return(loadGIF(filename)) ;
            break;
          
    case PNG:
            return(loadPNG(filename)) ;
            break;
            
    default:
		    return (false);
          
  }
  return( true );
}

/*--------------------------------------------------------------------------
                                              loadPNG
    PURPOSE: loads a PNG graphic file into a char map
    RETURN: true if loading was succesful else returns false.
---------------------------------------------------------------------------*/
bool imageLoader::loadPNG(char *n)
{
    FILE *f;
  
    png_struct* png_ptr;
    png_infop info_ptr;
    png_uint_32 w, h, channels;
    
    int bit_depth, color_type, filter_type, compression_type, interlace_type;

    if (loaded == true)
	    return(false);
	    
    f = fopen(n, "rb");
  
	if (f == 0)
		return false;
  
    png_ptr = new png_struct ;
    png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL );
  
    if ( !png_ptr ) {
	    fclose( f);  
	    return false ;
    }
  
    info_ptr = png_create_info_struct(png_ptr) ;
  
    if ( !info_ptr ) 
    {
      png_destroy_read_struct(&png_ptr, NULL, NULL);
      fclose(f);        
      return false ;
    }
      
    png_init_io(png_ptr, f);
    png_read_info(png_ptr, info_ptr);
  
    // retrieving image informations
    png_get_IHDR(png_ptr, info_ptr, &w, &h, &bit_depth, &color_type, &filter_type, &compression_type, &interlace_type);
    channels = png_get_channels(png_ptr, info_ptr);
  
    png_set_strip_16(png_ptr);
    png_set_expand(png_ptr);
    png_set_packing(png_ptr);

    png_set_gamma(png_ptr, 1.8, 0.50);

    png_set_interlace_handling(png_ptr);
    png_bytep row_pointers[h];
  
  
    for (unsigned int row = 0; row < h; row++) {
	    row_pointers[row] = (png_bytep) malloc(png_get_rowbytes(png_ptr, info_ptr));
    }
  
    png_read_image(png_ptr, row_pointers);
    png_read_end(png_ptr, info_ptr);
  
    png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
  
    fclose( f );
  
    width = w;
    height = h;
    size = width * height ;
    rvb_map = new unsigned char[size*4];
  
    switch(color_type)
    {
	    case PNG_COLOR_TYPE_RGB:
		      for ( unsigned int y = 0 ; y < height ; y++ )
			    for( unsigned int x = 0 ; x < width ; x++ )
			    {
				    rvb_map[( y * width*4) + (x*4)] = row_pointers[y][x*3];
				    rvb_map[( y * width*4) + (x*4)+1] = row_pointers[y][x*3 + 1];
				    rvb_map[( y * width*4) + (x*4)+2] = row_pointers[y][x*3 + 2];
				    rvb_map[( y * width*4) + (x*4)+3] = 255;
				 }
			  break;
			  
	    case PNG_COLOR_TYPE_RGB_ALPHA:
			  has_alpha = true;
		      for ( unsigned int y = 0 ; y < height ; y++ )
			    for( unsigned int x = 0 ; x < width ; x++ )
			    {
			      rvb_map[( y * width*4) + (x*4)] 	= row_pointers[y][x*4];	    // red
			      rvb_map[( y * width*4) + (x*4)+1] = row_pointers[y][x*4 + 1]; // green
			      rvb_map[( y * width*4) + (x*4)+2] = row_pointers[y][x*4 + 2]; // blue
			      rvb_map[( y * width*4) + (x*4)+3] = row_pointers[y][x*4 + 3]; // alpha
			    }
			  break;
		  
	    case PNG_COLOR_TYPE_GRAY:
			  has_alpha = true;
		      for ( unsigned int y = 0 ; y < height ; y++ )
			    for( unsigned int x = 0 ; x < width ; x++ )
			      {
				    rvb_map[( y * width*4) + (x*4)] = 0;
				    rvb_map[( y * width*4) + (x*4)+1] = 0;
				    rvb_map[( y * width*4) + (x*4)+2] = 0;
				    rvb_map[( y * width*4) + (x*4)+3] = row_pointers[y][x];
			      }
			  break;
			  
	    default:
			    cerr << "Unrecognized channel type, Loading canceled" << endl;
			    for (unsigned int row = 0; row < h; row++)
				    free(row_pointers[row]);

			    delete[] rvb_map;
			    return false;
			    break;
    }
    
    for (unsigned int row = 0; row < h; row++)
	    free( row_pointers[row] );
	    
    delete png_ptr;
    loaded = true;
    return(true);
}


/*--------------------------------------------------------------------------
                                              loadGIF
    PURPOSE: loads a GIF graphic file into a char map
    RETURN: true if loading was succesful else returns false.
---------------------------------------------------------------------------*/
bool imageLoader::loadGIF(char *n)
{
    FILE *f;
    int pix;
    unsigned int x, y; ;
    gdImage* gif_ptr;

    if (loaded == true)
	    return(false);
  
    f = fopen( n, "rb");
  
    if ( f == 0 )
	    return false;

    gif_ptr = new gdImage ;
    gif_ptr = gdImageCreateFromGif(f);
  
    fclose( f );
	
    width = gdImageSX(gif_ptr);
    height = gdImageSY(gif_ptr);
    size = width * height;

    rvb_map = new unsigned char[size*4];
  
    for ( y = 0 ; y < height ; y++ )
	    for ( x = 0; x < width ; x++)
	    {	
		    // get the color index of the pixel
		    pix = gdImageGetPixel(gif_ptr, x, y);
		    
		    // get the color from the color table with the index
		    rvb_map[ x*4 + y * width*4] = gif_ptr->red[pix];
		    rvb_map[ x*4 + 1 + y * width*4] = gif_ptr->green[pix];
		    rvb_map[ x*4 + 2 + y * width*4] = gif_ptr->blue[pix];
		    rvb_map[ x*4 + 3 + y * width*4] = 128 ; // default alpha channel
	    }	
    
    loaded = true;
    gdImageDestroy(gif_ptr);
    return(true);
}

/*--------------------------------------------------------------------------
                                              copyImageToCharmap
    PURPOSE: copies the pixel map into a given char array
    the callers needs to be sure the area are same size, no checks
    are made. Maybe later...
    RETURN: true if the dest area has been allocated else returns false.
---------------------------------------------------------------------------*/
bool imageLoader::copyImageToCharmap( unsigned char *dest, int t )
{
  if ( dest == 0 )  // destination buffer has not been allocated, bad !
    return (false);
  
  if ( t != type )
    setColorAlignement(t);

  memcpy( dest, rvb_map, size*4 );
  return( true) ;
}

/*--------------------------------------------------------------------------
                                              setColorAlignement
    PURPOSE: changes the color alignement of the pixel map, this means 
    reordering the ARGB info to match a particular architecture.
    RGBA is the default color alignement. For now we have RGBA for 
    openGL and BGRA for X windows
    WARNING CAN ONLY CHANGE ALIGNEMENT OF RGBA IMAGES, once the 
    alignement has been changed, it cannot be changed anymore, you
    need to call resetImage and load the image again.
    RETURN: nothing
---------------------------------------------------------------------------*/
bool imageLoader::setColorAlignement( int t )
{
  unsigned int x, y ;
  unsigned char a, r, g, b;


  if (type == t)
    return false;
    
  if (type != RGBA)
      return(false);
      
  // all this temp vars will be removed since everything works fine
  switch(t)
  {
      case(BGRA):
            for( y = 0 ; y < height ; y++)
                for( x = 0 ; x < width ; x++) {
			      r =	rvb_map[ x*4 + y * width*4] ;
			      g = rvb_map[ x*4 + 1 + y * width*4] ;
			      b = rvb_map[ x*4 + 2 + y * width*4] ;
			      a = rvb_map[ x*4 + 3 + y * width*4] ;
		  
			      rvb_map[ x*4 + y * width*4] = b;
			      rvb_map[ x*4 + 1 + y * width*4] = g;
			      rvb_map[ x*4 + 2 + y * width*4] = r;
			      rvb_map[ x*4 + 3 + y * width*4] = a;
                }
            break;
            
      case(RGBA):
            for( y = 0 ; y < height ; y++)
                for( x = 0 ; x < width ; x++) {
			      r =	rvb_map[ x*4 + y * width*4] ;
			      g = rvb_map[ x*4 + 1 + y * width*4] ;
			      b = rvb_map[ x*4 + 2 + y * width*4] ;
			      a = rvb_map[ x*4 + 3 + y * width*4] ;
			      
			      rvb_map[ x*4 + y * width*4] = r;
			      rvb_map[ x*4 + 1 + y * width*4] = g;
			      rvb_map[ x*4 + 2 + y * width*4] = b;
			      rvb_map[ x*4 + 3 + y * width*4] = a;
                }
      case(ARGB):
            for( y = 0 ; y < height ; y++)
                for( x = 0 ; x < width ; x++) {
			      r =	rvb_map[ x*4 + y * width*4] ;
			      g = rvb_map[ x*4 + 1 + y * width*4] ;
			      b = rvb_map[ x*4 + 2 + y * width*4] ;
			      a = rvb_map[ x*4 + 3 + y * width*4] ;
		  
			      rvb_map[ x*4 + y * width*4] = r;
			      rvb_map[ x*4 + 1 + y * width*4] = g;
			      rvb_map[ x*4 + 2 + y * width*4] = b;
			      rvb_map[ x*4 + 3 + y * width*4] = a;
                }
		
        default:
                break;
    }
    return(true);
}

/*--------------------------------------------------------------------------
                                              hasAlpha
    PURPOSE: returns image information
    RETURN: true if the image has an alpha channel otherwise false
---------------------------------------------------------------------------*/
bool imageLoader::hasAlpha(void)
{
    return(has_alpha);
}

/*--------------------------------------------------------------------------
                                              setAlpha
    PURPOSE: sets an uniform alpha level for the whole image
    RETURN: nothing
---------------------------------------------------------------------------*/
void imageLoader::setAlpha(unsigned char a)
{
    if (loaded == false)
	    return;

    // this stupid test per pixel will be removed, debugging purpose
    for ( unsigned int y = 0 ; y < height ; y++ )
	    for( unsigned int x = 0 ; x < width ; x++ )
	    {
		    switch(type) 
		    {
			    case ARGB:
				      rvb_map[( y * width*4) + (x*4)] = a;
				      break;
					    
			    default:
				     rvb_map[( y * width*4) + (x*4)+3] = a;
				     break;
		     }
	    }
}

