/*---------------------------------------------------------------------------

                                                      Image Loader v1.0
  Eneko Bilbao 1999
---------------------------------------------------------------------------*/

/* STATUS
    -   Loads perfectly GIF files.
    -		Loads perfectly PNG files (RGB, RGB+ALPHA, GRAYSCALE).
    -   JPEG support removed
*/

/* TODO
    -		Remove the stupid test in setAlpha
    -		Add a constructor with all parameters
    -		Add JPEG support
    -   Maybe add some transformation like scaling because the texture's
	    dimensions must be a power of 2 and cannot exced 256x256 for 3DFX hardware.
*/    

/* BUGS
    -		none found in the Linux version
    -		in the win32 version, PNG loading crashes, need to check to win32 libPNG sources
*/
    
#ifndef _IMAGE_LOADER_
#define _IMAGE_LOADER_

#include <iostream.h>

extern "C" {
    
    #include <string.h>
    #include <stdlib.h>
    #include <stdio.h>
// if you use a thing called grtoul, replace the above line with:     #include <grtoul.h>
    #include <gd.h>
    #include <png.h>
}

// file types
#define GIF   0
#define PNG   1

// color alignement
#define ARGB 0          // default color to int alignement 
#define BGRA 1          // XImage color alignement ( for X window systems)
#define RGBA 2          // openGL and this class default color alignement


class imageLoader {
  
  private:
        int 						type;                 // refers to the color alignement
        unsigned int 			width, height, size;  // image parameters
	    bool 					loaded;               // whether it's already loaded or not
	    bool 					has_alpha;            // whether the image has an alpha channel or not
        unsigned char 	*rvb_map;    										// the pixel map
        
  public:
        imageLoader(void)
        { 
          width = height = 0;
          rvb_map = 0;
          loaded = false;
	      has_alpha = false;
          type = RGBA;
        }
        
        ~imageLoader(void)
        {
            if ( rvb_map != 0)
                delete[] rvb_map ;
        }
        
        bool    loadImage(int t, char *filename);
       
        bool    loadGIF(char *filename);
        bool    loadPNG(char *filename);
                                    
        bool    resetImage(void);
        bool    copyImageToCharmap(unsigned char *dest, int t=RGBA); // t refers to the color alignement 
        
        int     getColorAlignement(void);
        bool    setColorAlignement(int);
	    
	    void			setAlpha(unsigned char);
	    bool			hasAlpha(void);
	    
	    int			getWidth(void);
	    int			getHeight(void);
	    int			getSize(void);
};

#endif /*--------------- End of class definition --------------------------*/
