/*---------------------------------------------------------------------------

												gl Billard					   v1.0 
  Eneko Bilbao 1999
---------------------------------------------------------------------------*/

/* Comment
    sometimes you might read "executes an object", I use this because I
    refer to the thread paradigm and I hope to be able to add it soon.
    So executing an object is running its run method (implemented in Model class
*/

/*    main file
    initializes the render device,
	    launches the demo,
	    launches the game,
	    check collisions
*/
    
/* STATUS
    -   collision test is nearly working
    -		demo mode ok
    -   no memory leak
    -	  scores and player turn not implemented
*/

/* TODO
    -		optimize all cos and sin computations since they come up often and are
	    very expensive.
    -		add a dedicated object for game play(scores, player turns).
    -		lighmaps.
    -		shadows
    -		threading
    -		better graphics.
    -		make a Be and Windows port.    
*/    

/* BUGS
    -		collision between ball 0 and 2 still not working
*/



#ifndef _MAIN_H_
#define _MAIN_H_

#include "globals.h"

/*- enables or disables special effects -*/
#define HIGH_QUALITY     0
#define NORMAL_QUALITY   1
#define LOW_QUALITY      2

static int image_quality = NORMAL_QUALITY;

/*----------------------------------*
 *   Globals                        *
 *----------------------------------*/
linked_list<Texture*> map_list;
linked_list<Actor*> object_list;

/*- object counter -*/
static int object_count = 0;
static int game_mode = 0;

/*- game turns and scores -*/
static int players_turn = 0;

static float timer, time_slice;
static float cam_rot_fact = 720.0 / SCREEN_WIDTH ;
static float queue_rot_fact = 720.0 / SCREEN_WIDTH ;
static float force = 0.0;

Camera eye(0,0,1.0);
static float cam[3];
 
/*- Framerate -*/
static long start = 0,end = 0, frame = 0;
static long elapsed = 0;

/*- mouse handling -*/
static int cam_mov = 1;

static void Key( unsigned char key, int x, int y );
static void SpecialKey( int key, int x, int y );
static void motion(int x, int y);
static void mouse(int button, int state, int x, int y);

void init_game_engine(void);

void init_demo(void);
void run_demo(void);

void load_textures(void);
void load_objects(void);

void init_game(void);
void run_game(void);

void retarget_queue(int);
void hit_ball(void);
void checkCollision(void);
void impact(int , int);

void update_score(int, int );
void check_score(int, int);
void check_turn(void);

void set_camera();
void set_lighting(void);
void run_world(void);

void switch_demo_to_game(void);
void switch_light_on(GLenum which_one);

int originalWindow = 0, currentWindow;

int check_mode( char *mode)
{
    glutGameModeString(mode);
    cerr << mode << ", your system says : " ;
    
    if ( glutGameModeGet(GLUT_GAME_MODE_POSSIBLE) != 0 ) { 
	    cerr << "Yes !" << endl ;
	    return 1;
    }
    
    cerr << "no..." << endl;
    return 0;
}
	    
void query_system(void)
{
	cerr << "Querying System..." << endl;
	cerr << "GL_VENDOR:   " << (char*)glGetString(GL_VENDOR)   << endl;
	cerr << "GL_RENDERER: " << (char*)glGetString(GL_RENDERER) << endl;
    cerr << "GL_VERSION:  " << (char*)glGetString(GL_VERSION)  << endl ;	
    cerr << "---------------------------------------------"    << endl;
    cerr << "Trying to use new glutGameMode, checking available modes:" << endl;
    
    /*
    check_mode("1024x768:16@75");
    check_mode("1024x768:16@72");
    check_mode("1024x768:16@70");
    */
    check_mode("1024x768:16@80");    
    check_mode("1024x768:16@79");    
}



/*- Objects indexes -*/
#define BLANCHE            0
#define ROUGE              1
#define BLEU               2
#define QUEUE              3
#define OMBRE_BLANCHE      4
#define OMBRE_ROUGE        5
#define OMBRE_BLEU         6


/*- index of the texture in the list -*/
#define TEX_GROUND        0
#define TEX_WALL_1        1
#define TEX_WALL_2        2
#define TEX_ROOF          3
#define TEX_SPOTLIGHT     4
#define TEX_PAINTING      5
#define TEX_INTRO         6
#define TEX_LIGHTMAP      7
#define TEX_PAINTING_2    8


/*-----------------------------------------------------------
    display_lists
  ---------------------------------------------------------*/    
  
/*- Display Lists indexes -*/
#define ROOM            1
#define TABLE           2
#define LIGHT_MAP       3
#define COMPTOIR        4

void make_room(void);
void make_billard(void);
void make_lightmap(void);
void make_bank(void);

// for mapping coordinates auto-generation
static GLfloat map_plane[] = { 16.0, 1.0, 1.0 };

void make_room(void)
{
    glNewList(ROOM, GL_COMPILE_AND_EXECUTE);
   
    // ground
    glPushMatrix();
    
	    glTranslatef(-8.0, -3.0, -14.0 );
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
        
        map_list[TEX_GROUND]->set();
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        glBindTexture(GL_TEXTURE_2D, map_list[TEX_GROUND]->getRef());
	
		for(int z = 0 ; z < 7 ; z++)
		{
		    for(int x = 0 ; x < 4 ; x++)
		    {
			    glBegin(GL_QUADS);
				    glNormal3f(0.0, 1.0, 0.0);
				    
				    glTexCoord2f(4.0, 4.0);
				    glVertex3f( 4.0, 0.0,  4.0 );
            
				    glTexCoord2f(4.0, 0.0);
				    glVertex3f( 4.0, 0.0,  0.0 );
            
				    glTexCoord2f(0.0, 0.0);
				    glVertex3f( 0.0, 0.0,  0.0 );
            
				    glTexCoord2f(0.0, 4.0);
				    glVertex3f( 0.0, 0.0, 4.0 );
			    glEnd();
			    glTranslatef(4.0, 0.0, 0.0);
		    }
			glTranslatef(-16.0, 0.0, 4.0);
		}
    glPopMatrix();
    
    // tapis
    glPushMatrix();
        glTranslatef(0.0, -2.70, 4.0 );
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_light_blue);
       	glDisable(GL_TEXTURE_2D);
				
        glTranslatef(-4.0, 0.0, 0.0 );
        glBegin(GL_QUADS);
            glNormal3f(0.0, 1.0, 0.0);
            glVertex3f( 0.0, 0.0,  0.0 );
            glVertex3f( 8.0, 0.0,  0.0 );
            glVertex3f( 8.0, 0.0,  -8.0 );
            glVertex3f( 0.0, 0.0, -8.0 );
        glEnd();
    glPopMatrix();


	// plafond
	glPushMatrix();
	    glTranslatef(-8.0, 3.0, -14.0 );
        
        map_list[TEX_ROOF]->set();
		
	    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
	    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
		glBindTexture(GL_TEXTURE_2D, map_list[TEX_ROOF]->getRef());
		
		for(int z = 0 ; z < 7 ; z++)
		{
		    for(int x = 0 ; x < 4 ; x++)
		    {
			    glBegin(GL_QUADS);
				    glNormal3f(0.0, -1.0, 0.0);
				    
				    glTexCoord2f(0.0, 2.0);
				    glVertex3f( 0.0, 0.0, 4.0 );
				    
				    glTexCoord2f(0.0, 0.0);
				    glVertex3f( 0.0, 0.0,  0.0 );

	    				glTexCoord2f(2.0, 0.0);
				    glVertex3f( 4.0, 0.0,  0.0 );
            
				    glTexCoord2f(2.0, 2.0);
				    glVertex3f( 4.0, 0.0,  4.0 );
            
			    glEnd();
			    glTranslatef(4.0, 0.0, 0.0);
		    }
			glTranslatef(-16.0, 0.0, 4.0);
			
			// une poutre
			glPushMatrix();
				    map_list[TEX_WALL_2]->set();
				    glBindTexture(GL_TEXTURE_2D, map_list[TEX_WALL_2]->getRef());
				    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);

				    glEnable(GL_TEXTURE_GEN_S);
				    glEnable(GL_TEXTURE_GEN_T);
			    
				    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
				    glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
			    
				    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
				    glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
			    
				    glTranslatef( 8.0, 0.0, 0.0);
				    glScalef(16.0, 0.2, 0.2);
				    glutSolidCube(1);
			    
				    glDisable(GL_TEXTURE_GEN_S);
				    glDisable(GL_TEXTURE_GEN_T);
			glPopMatrix();    
		}
	    glPopMatrix();

    // left Wall
    glPushMatrix();
    
        map_list[TEX_WALL_1]->set();

        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
		glBindTexture(GL_TEXTURE_2D, map_list[TEX_WALL_1]->getRef());
		
	    glTranslatef(-8.0, 0.0, -14.0 );
	    
		for(int y = 0 ; y < 2 ; y++)
		{
		    for(int z = 0 ; z < 7 ; z++)
		    {
			    glBegin(GL_QUADS);
				    
				    glNormal3f(1.0,0.0, 0.0);
				    
				    glTexCoord2f(0.0, 0.0);
				    glVertex3f( 0.0, 3.0,  0.0 );

	    				glTexCoord2f(1.0, 0.0);
				    glVertex3f( 0.0, 3.0,  4.0 );
            
				    glTexCoord2f(1.0, 1.0);
				    glVertex3f( 0.0, 0.0,  4.0 );
				    
				    glTexCoord2f(0.0, 1.0);
				    glVertex3f( 0.0, 0.0, 0.0 );
				    
			    glEnd();
			    glTranslatef(0.0, 0.0, 4.0);
		    }
			glTranslatef(0.0, -3.0, -28.0);
		}
	    glPopMatrix();
	    
    // right Wall
    glPushMatrix();
    
        map_list[TEX_WALL_1]->set();
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        glBindTexture(GL_TEXTURE_2D, map_list[TEX_WALL_1]->getRef());

	    glTranslatef( 8.0, 3.0, -14.0 );
	    
		for(int y = 0 ; y < 2 ; y++)
		{
		    for(int z = 0 ; z < 7 ; z++)
		    {
			    glBegin(GL_QUADS);
				    glNormal3f( -1.0, 0.0, 0.0);
					
				    glTexCoord2f(0.0, 0.0);
					glVertex3f( 0.0, 0.0, 0.0 );
				    
					glTexCoord2f(0.0, 1.0);
				    glVertex3f( 0.0,-3.0, 0.0 );

				    glTexCoord2f(1.0, 1.0);
				    glVertex3f( 0.0,-3.0, 4.0 );

				    glTexCoord2f(1.0, 0.0);
				    glVertex3f( 0.0, 0.0, 4.0 );
				    
			    glEnd();
			    glTranslatef(0.0, 0.0, 4.0);
		    }
		    glTranslatef( 0.0, -3.0, -28.0 );
		}
    glPopMatrix();

    // une bande de lumiere
    glPushMatrix();
        glTranslatef( 0.0, 2.975, -16.0 );
		
		glScalef( 0.5, 1.0, 0.5);
		
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
        map_list[TEX_SPOTLIGHT]->set();
		
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	    glBindTexture(GL_TEXTURE_2D, map_list[TEX_SPOTLIGHT]->getRef());
	
			for(int i=0 ; i < 7 ; i++) {
			
				glTranslatef(0.0, 0.0, 8.0 );
				
			    glBegin(GL_QUADS);
					glNormal3f( 0.0, -1.0, 0.0);
					
		            glTexCoord2f(0.0, 0.0);
				    glVertex3f( 0.0, 0.0, 0.0 );
				    
					glTexCoord2f(0.0, 1.0);
				    glVertex3f( 0.0, 0.0, -1.0 );

				    glTexCoord2f(1.0, 1.0);
				    glVertex3f( 1.0, 0.0, -1.0 );

					glTexCoord2f(1.0, 0.0);
				    glVertex3f( 1.0, 0.0, 0.0 );
			    glEnd();
			}
	glPopMatrix();

    // l'affiche family pool
    glPushMatrix();
        glTranslatef( -2.0, 1.0, -13.7 );
	    map_list[TEX_PAINTING]->set();

	    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        glBindTexture(GL_TEXTURE_2D, map_list[TEX_PAINTING]->getRef());
        
        glBegin(GL_QUADS);
            glNormal3f( 0.0, 0.0, 1.0);
            
            glTexCoord2f(0.0, 0.0);
            glVertex3f( -4.0, 2.0, 0.0 );
            
            glTexCoord2f(0.0, 1.0);
            glVertex3f( -4.0, -2.0, 0.0 );
            
            glTexCoord2f(1.0, 1.0);
            glVertex3f( 4.0, -2.0, 0.0 );
            
            glTexCoord2f(1.0, 0.0);
            glVertex3f( 4.0, 2.0, 0.0 );
        glEnd();
    glPopMatrix();

    // l'affiche du barbier
    glPushMatrix();
	    
        glTranslatef( -7.7, 0.0, 0.0);
	    glRotatef(90.0, 0.0, 1.0, 0.0);
	    map_list[TEX_PAINTING]->set();

	    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        glBindTexture(GL_TEXTURE_2D, map_list[TEX_PAINTING_2]->getRef());
        
        glBegin(GL_QUADS);
            glNormal3f( 0.0, 0.0, 1.0);
            
            glTexCoord2f(0.0, 0.0);
            glVertex3f( -1.0, 1.0, 0.0 );
            
            glTexCoord2f(0.0, 1.0);
            glVertex3f( -1.0, -1.0, 0.0 );
            
            glTexCoord2f(1.0, 1.0);
            glVertex3f( 1.0, -1.0, 0.0 );
            
            glTexCoord2f(1.0, 0.0);
            glVertex3f( 1.0, 1.0, 0.0 );
        glEnd();
    glPopMatrix();

    // Back Wall
    glPushMatrix();
	    map_list[TEX_WALL_1]->set();
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        glBindTexture(GL_TEXTURE_2D, map_list[TEX_WALL_1]->getRef());

		glTranslatef( -12.0, 6.0, -14.0 );
		
		for(int j=0 ; j < 2 ; j++) {
			glTranslatef(0.0, -3.0, 0.0 );
		    for(int i=0 ; i < 4 ; i++) {
				glTranslatef(4.0, 0.0, 0.0 );
				glBegin(GL_QUADS);
					glNormal3f( 0.0, 0.0, 1.0);
					
		            glTexCoord2f(0.0, 0.0);
				    glVertex3f( 0.0, 0.0, 0.0 );
					
					glTexCoord2f(0.0, 1.0);
				    glVertex3f( 0.0,-3.0, 0.0 );
					
					glTexCoord2f(1.0, 1.0);
				    glVertex3f( 4.0,-3.0, 0.0 );
					
					glTexCoord2f(1.0, 0.0);
				    glVertex3f( 4.0, 0.0, 0.0 );
				glEnd();					
			}
			glTranslatef(-16.0, 0.0, 0.0 );
		}
    glPopMatrix();
    
    // Front Wall
    glPushMatrix();
		map_list[TEX_WALL_1]->set();
	
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
        glBindTexture(GL_TEXTURE_2D, map_list[TEX_WALL_1]->getRef());

        glTranslatef( -12.0, 6.0, 14.0 );

			glTranslatef(0.0, -3.0, 0.0 );
		    for(int i=0 ; i < 4 ; i++) {
				glTranslatef(4.0, 0.0, 0.0 );
				glBegin(GL_QUADS);
					glNormal3f( 0.0, 0.0, -1.0);
					
		            glTexCoord2f(0.0, 0.0);
				    glVertex3f( 0.0, 0.0, 0.0 );
					
					glTexCoord2f(1.0, 0.0);
				    glVertex3f( 4.0, 0.0, 0.0 );
				    
				    
					glTexCoord2f(1.0, 1.0);
				    glVertex3f( 4.0,-3.0, 0.0 );

				    glTexCoord2f(0.0, 1.0);
				    glVertex3f( 0.0,-3.0, 0.0 );
				glEnd();					
			}
			
			glTranslatef(-16.0, 0.0, 0.0 );
			
			glPushMatrix();
			    map_list[TEX_WALL_2]->set();
			    
			    glBindTexture(GL_TEXTURE_2D, map_list[TEX_WALL_2]->getRef());
			    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);

			    glEnable(GL_TEXTURE_GEN_S);
			    glEnable(GL_TEXTURE_GEN_T);
			    
			    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
			    glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
			    
			    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
			    glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
			    
			    glTranslatef( 12.0, -3.0, 0.0);
			    glScalef(16.0, 0.2, 0.2);
			    glutSolidCube(1);
			    
			    glDisable(GL_TEXTURE_GEN_S);
			    glDisable(GL_TEXTURE_GEN_T);
			glPopMatrix();    
			
			glTranslatef(0.0, -3.0, 0.0 );
		    for(int i=0 ; i < 4 ; i++) {
				glTranslatef(4.0, 0.0, 0.0 );
				glBegin(GL_QUADS);
					glNormal3f( 0.0, 0.0, -1.0);
					
		            glTexCoord2f(0.0, 0.0);
				    glVertex3f( 0.0, 0.0, 0.0 );
					
					glTexCoord2f(1.0, 0.0);
				    glVertex3f( 4.0, 0.0, 0.0 );
				    
				    
					glTexCoord2f(1.0, 1.0);
				    glVertex3f( 4.0,-3.0, 0.0 );

					
				    glTexCoord2f(0.0, 1.0);
				    glVertex3f( 0.0,-3.0, 0.0 );
				glEnd();					
			}
	
    glPopMatrix();
    glDisable(GL_TEXTURE_2D);
    glEndList();
}

void make_billard(void)
{
    glNewList(TABLE, GL_COMPILE);
    
    glPushMatrix();
		glTranslatef( -2.5, -1.0, -5.0);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_light_green);
	    glScalef(0.5, 0.5, 0.5);
    	
      // plateau
	glBegin(GL_QUADS);
      	glNormal3f( 0.0, 1.0, 0.0);
        
        glVertex3f( 0.0, 0.0, 0.0);		// up left
        glVertex3f( 0.0, 0.0, 14.5); // down left
      	glVertex3f( 9.0, 0.0, 14.5);  // down right
        glVertex3f( 9.0, 0.0, 0.0);   // up	right
      
      	// fond haut
      	glNormal3f( 0.0, 0.0, -1.0);
			  
        glVertex3f( 0.0, 0.0, 0.0);		// up left
        glVertex3f( 9.0, 0.0, 0.0); // down left
      	glVertex3f( 9.0, 0.2, 0.0);  // down right
        glVertex3f( 0.0, 0.2, 0.0);   // up	right
      
 	     // fond bas
      	glNormal3f( 0.0, 0.0, 1.0);
        
        glVertex3f( 0.0, 0.0, 14.5);		// up left
        glVertex3f( 0.0, 0.2, 14.5);   	// up	right
        glVertex3f( 9.0, 0.2, 14.5);  	// down right
        glVertex3f( 9.0, 0.0, 14.5); 		// down left
      
      // fond gauche
      	glNormal3f( 1.0, 0.0, 0.0);
        
        glVertex3f( 0.0, 0.0, 0.0);		// up left
        glVertex3f( 0.0, 0.2, 0.0);   	// up	right
        glVertex3f( 0.0, 0.2, 14.5);  	// down right
        glVertex3f( 0.0, 0.0, 14.5); 		// down left
      
      // fond droit
      	glNormal3f( -1.0, 0.0, 0.0);
        
        glVertex3f( 9.0, 0.0, 0.0);		// up left
        glVertex3f( 9.0, 0.0, 14.5); 		// down left
        glVertex3f( 9.0, 0.2, 14.5);  	// down right
        glVertex3f( 9.0, 0.2, 0.0);   	// up	right
      
      //rebords haut
      glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_dark_red);
      	glNormal3f( 0.0, 1.0, 0.0);
        
        glVertex3f( 0.0, 0.2,-0.5);		// up left
        glVertex3f( 0.0, 0.2, 0.0); 	// down left
        glVertex3f( 9.0, 0.2, 0.0);  	// down right
        glVertex3f( 9.0, 0.2,-0.5);   // up	right
      
      //rebords bas
      	glNormal3f( 0.0, 1.0, 0.0);
        
        glVertex3f( 0.0, 0.2, 14.0);		// up left
        glVertex3f( 0.0, 0.2, 14.5); 	// down left
        glVertex3f( 9.0, 0.2, 14.5);  	// down right
        glVertex3f( 9.0, 0.2, 14.0);   // up	right
      
      //rebords gauche
      	glNormal3f( 0.0, 1.0, 0.0);
        
        glVertex3f(-0.5, 0.2, -0.5);		// up left
        glVertex3f(-0.5, 0.2, 14.5); 	// down left
        glVertex3f( 0.0, 0.2, 14.5);  	// down right
        glVertex3f( 0.0, 0.2, -0.5);   // up	right

      //rebords droit
      	glNormal3f( 0.0, 1.0, 0.0);
        
        glVertex3f( 9.0, 0.2, -0.5);		// up left
        glVertex3f( 9.0, 0.2, 14.5); 		// down left
        glVertex3f( 9.5, 0.2, 14.5);  	// down right
        glVertex3f( 9.5, 0.2, -0.5);   	// up	right
      
      //panneau avant
      	glNormal3f( 0.0, 0.0, 1.0);
        
        glVertex3f(-0.5,  0.2, -0.5);		// up left
        glVertex3f( 9.5,  0.2, -0.5); 		// down right
        glVertex3f( 9.5, -2.3, -0.5);  		// down right
        glVertex3f(-0.5, -2.3, -0.5);   // up	left
      
      //panneau arriere
      	glNormal3f( 0.0, 0.0, 1.0);
        
        glVertex3f(-0.5,  0.2, 14.5);		// up left
		glVertex3f(-0.5, -2.3, 14.5);   // up	right        
        glVertex3f( 9.5, -2.3, 14.5);  	// down right
        glVertex3f( 9.5,  0.2, 14.5); 	// down left
      
      //panneau gauche
      	glNormal3f( -1.0, 0.0, 0.0);
        glVertex3f(-0.5,  0.2,-0.5);		// up left
		
		glVertex3f(-0.5, -2.3,-0.5);   // up	right        
        glVertex3f(-0.5, -2.3,14.5);  	// down right
        glVertex3f(-0.5,  0.2,14.5); 	// down left
      
      //panneau droite
      	glNormal3f( 1.0, 0.0, 0.0);
        
        glVertex3f( 9.5,  0.2, -0.5);		// up left
		glVertex3f( 9.5,  0.2, 14.5); 	// down left
        glVertex3f( 9.5, -2.3, 14.5);  	// down right        
		glVertex3f( 9.5, -2.3, -0.5);   // up	right        
	glEnd();
	      
		//      les 4 pieds 
		// pied haut gauche
    glPushMatrix();
	    glTranslatef( 0.0, -2.65, 0.0);
	    glScalef( 1.0, 2.0, 1.0);
  	    glutSolidCube(1.0);
    glPopMatrix();
      
			// pied bas gauche 
    glPushMatrix();
	    glTranslatef(0.0, -2.65, 14.0);
	    glScalef( 1.0, 2.0, 1.0);
	    glutSolidCube(1.0);
    glPopMatrix();
      
			// pied bas droit
    glPushMatrix();
		glTranslatef(9.0, -2.65, 14.0);
		glScalef( 1.0, 2.0, 1.0);
  	    glutSolidCube(1.0);
    glPopMatrix();
      
    	// pied haut droit
      glPushMatrix();
		glTranslatef(9.0, -2.65, 0.0);
		glScalef( 1.0, 2.0, 1.0);
  	    glutSolidCube(1.0);
      glPopMatrix();
      
    glPopMatrix();
    glEndList();
}

void make_bank(void)
{
    map_plane[0] = 4.0;
    map_plane[1] = 1.0;
    map_plane[2] = 1.0;
    
    glNewList(COMPTOIR, GL_COMPILE);
	    glPushMatrix();
	    
		    map_list[TEX_WALL_2]->set();
			    
			glBindTexture(GL_TEXTURE_2D, map_list[TEX_WALL_2]->getRef());
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);

		    glPushMatrix();
			    glTranslatef(-1.0, -1.6, -10.0);
			    glScalef(14.0, 2.8, 1.0);
			    
			    glEnable(GL_TEXTURE_GEN_S);
			    glEnable(GL_TEXTURE_GEN_T);
			    
			    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
				glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
			    
			    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
			    glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
			    
			    glutSolidCube(1);
			    
			    glDisable(GL_TEXTURE_GEN_S);
			    glDisable(GL_TEXTURE_GEN_T);
			    
			    glPushMatrix();
				    glScalef(1/16.0, 1/2.8, 1/0.8);
				    glTranslatef(0.0, 1.4, 0.0);
				    glScalef(16.0, 0.2, 1.2);

				    map_plane[0] = 1.0;
				    map_plane[1] = 0.0;
				    map_plane[2] = 1.0;
				    
				    glEnable(GL_TEXTURE_GEN_S);
				    glEnable(GL_TEXTURE_GEN_T);
			    
				    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
				    glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
			    
					glTexGeni(GL_T, GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
					glTexGenfv(GL_S, GL_OBJECT_PLANE,map_plane);
				    
				    glutSolidCube(1);
			    glPopMatrix();
		    glPopMatrix();
		    
		    glDisable(GL_TEXTURE_GEN_S);
		    glDisable(GL_TEXTURE_GEN_T);
		    glDisable(GL_TEXTURE_2D);
		    
		glPopMatrix();
	glEndList();
}
	
void make_lightmap(void)
{
	glNewList(LIGHT_MAP, GL_COMPILE);
	    glPushMatrix();
	    
	        glDisable(GL_LIGHTING);
		    glDisable(GL_DEPTH);
		    glDisable(GL_CULL_FACE);
		    
		    glEnable(GL_TEXTURE_2D);
		    glEnable(GL_BLEND);
		    glDepthFunc(GL_LEQUAL);
		    
			glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_white);
			
		    glBlendFunc(GL_ZERO, GL_SRC_COLOR);
		    
		    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		    glBindTexture(GL_TEXTURE_2D, map_list[TEX_LIGHTMAP]->getRef());
		    
		    glBegin(GL_QUADS);
			    glNormal3f( 0.0, 0.0, -1.0);
            
			    glTexCoord2f(0.0, 0.0);
			    glVertex3f( 0.0, 0.0, 0.0 );
            
			    glTexCoord2f(0.0, 2.0);
			    glVertex3f( 0.0, -6.0, 0.0 );
            
			    glTexCoord2f(4.0, 2.0);
			    glVertex3f( 16.0, -6.0, 0.0 );
            
			    glTexCoord2f(4.0, 0.0);
			    glVertex3f( 16.0, 0.0, 0.0 );
		    glEnd();
		
		    glDisable(GL_TEXTURE_2D);
		    glDisable(GL_BLEND);

			glEnable(GL_LIGHTING);
		    glEnable(GL_DEPTH);
		    glEnable(GL_CULL_FACE);

	    glPopMatrix();
    glEndList();
}  

#endif

