#include "model.h"

/*--------------------------------------------------------
    function: setSpeed
    parameters: 3 coordinates x, y, z
    purpose:	 sets the speed of the object
  -------------------------------------------------------*/
void    Model::setSpeed(float a, float b, float c)
{
    spd.x = a ;
    spd.y = b ;
    spd.z = c ;
}

/*--------------------------------------------------------
    function: setSpeed
    parameters: a vertex
    purpose:	 sets the speed of the object
  -------------------------------------------------------*/
void    Model::setSpeed(vertex& v)
{
    spd = v ;
}

/*--------------------------------------------------------
    function: setPivot
    parameters: 3 coordinates x, y, z
    purpose:	 sets the barycenter or pivot of the object
         		   by default its the object ieself
  -------------------------------------------------------*/
void    Model::setPivot(float a, float b, float c)
{
    piv.x = a ;
    piv.y = b ;
    piv.z = c ;
}

/*--------------------------------------------------------
    function: setPivot
    parameters: a vertex
    purpose:	 sets the barycenter or pivot of the object
         		   by default its the object ieself
  -------------------------------------------------------*/
void    Model::setPivot(vertex& v)
{
    piv = v ;
}

/*--------------------------------------------------------
    function: setPosition
    parameters: 3 coordinates x, y, z
    purpose:	  sets the position of the object in 3d space
  -------------------------------------------------------*/
void    Model::setPosition(float a, float b, float c)
{
    pos.x = a ; 
    pos.y = b ;
    pos.z = c ;
}

/*--------------------------------------------------------
    function: setPosition
    parameters: a vertex
    purpose:	  sets the position of the object in 3d space
  -------------------------------------------------------*/
void    Model::setPosition(vertex& v)
{
    pos = v ;
}

/*--------------------------------------------------------
    function: setRotation
    parameters: 3 coordinates x, y, z
    purpose:	  sets the rotation of the object in 3d space
			    in degrees.
  -------------------------------------------------------*/
void	Model::setRotation(float a, float b, float c)
{
    rot.x = a ;
    rot.y = b ;
    rot.z = c ;
}

/*--------------------------------------------------------
    function: setRotation
    parameters: a vertex
    purpose:	  sets the rotation of the object in 3d space
			    in degrees.
  -------------------------------------------------------*/
void	Model::setRotation(vertex& v)
{
    rot = v;
}

/*--------------------------------------------------------
    function: setRotationSpeed
    parameters: 3 coordinates x, y, z
    purpose:	  sets the rotation speed of the object in 
			   3d space, in degrees.
  -------------------------------------------------------*/
void	Model::setRotationSpeed(float a, float b, float c)
{
    rspd.x = a ;
    rspd.y = b ;
    rspd.z = c ;
}
        
/*--------------------------------------------------------
    function: setRotationSpeed
    parameters: a vertex
    purpose:	  sets the rotation of the object in 3d space
			   in degrees.
  -------------------------------------------------------*/
void	Model::setRotationSpeed(vertex& v)
{
    rspd = v ;
}

/*--------------------------------------------------------
    function: setMotion
    parameters: 3 coordinates x, y, z
				0.0 does not modify the previous speed, 
				use the vertex version to override all 
				coordinates
    purpose:	  sets the speed of the object in 3d space, 
			   in degrees.
  -------------------------------------------------------*/
void	Model::setMotion(float a, float b, float c)
{
    if ( a != 0 )
	    spd.x = a ;
    if ( b != 0 )
	    spd.y = b ;
	if ( c !=0 )    
	    spd.z = c ;
}

/*--------------------------------------------------------
    function: setMotion
    parameters: a vertex
				overrides all coordinates
    purpose:	  sets the speed of the object in 3d space, 
			   in degrees.
  -------------------------------------------------------*/
void	Model::setMotion(vertex& v)
{
    spd = v;
}    

/*--------------------------------------------------------
    function: Move
    parameters: 3 coordinates x, y, z
			   that are added to the current speed.
			   give no parameter tu use only current speed.
    purpose:	  moves the object
			   decelerates
			   checks for collisions and bouncing.
  -------------------------------------------------------*/
void	Model::Move(float a, float b, float c)
{
    pos.x += spd.x + a ;
    pos.y += spd.y + b ;
    pos.z += spd.z + c ;

    if ( bound_check == false )
	    return;
	    
//deceleration in x y z directions
    if ((spd.x < friction)&&( spd.x > -friction))
      spd.x = 0.0;
    else
    {
      if(spd.x > friction)
         spd.x -= friction;
      
      if(spd.x < -friction)
         spd.x += friction;
    }
    
    if ((spd.z < friction)&&( spd.z > -friction))
      spd.z = 0.0;
    else
    {
      if(spd.z > friction)
         spd.z -= friction;
    
		 if(spd.z < -friction)
         spd.z += friction;
   }
   	
    if ((spd.y < friction)&&( spd.y > -friction))
      spd.y = 0.0;
    else
    {
      if(spd.y > friction)
         spd.y -= friction;
      
      if(spd.y < -friction)
         spd.y += friction;
    }

    // rebond contre une parois
    if ((pos.x < top_left.x)||(pos.x > btm_right.x) )
    {
      spd.x *= -1.0;
      
      if (pos.x < top_left.x)
	      pos.x = top_left.x;
	      
      if (pos.x > btm_right.x)
	      pos.x = btm_right.x;
    }
    
    if ((pos.z < top_left.z)||(pos.z > btm_right.z) )
    {
      spd.z *= -1.0;
      
      if (pos.z < top_left.z)
	      pos.z = top_left.z;
	      
      if (pos.z > btm_right.z)
	      pos.z = btm_right.z;
    }

    if ((pos.y < top_left.y)||(pos.y > btm_right.y) )
    {
      spd.y *= -1.0;
      
      if (pos.y < top_left.y)
	      pos.y = top_left.y;
	      
      if (pos.y > btm_right.y)
	      pos.y = btm_right.y;
    }

}

/*--------------------------------------------------------
    function: Rotate
    parameters: 3 coordinates x, y, z
				that are added to the current rotation.
    purpose:	  rotates the object in 3d space, in degrees.
  -------------------------------------------------------*/
void	Model::Rotate(float a, float b, float c)
{
    rot.x += rspd.x + a ;
    rot.y += rspd.y + b ;
    rot.z += rspd.z + c ;
}
			
/*--------------------------------------------------------
    function: setMesh
    parameters: a predefined mesh type
    purpose:	  defines the kind of mesh for the object,
  -------------------------------------------------------*/
void	Model::setMesh(int m)
{
    mesh_type = m;
}

/*--------------------------------------------------------
    function: getMesh
    purpose:	  returns the kind of mesh for the object,
    return: the type of the mesh
  -------------------------------------------------------*/
int	Model::getMesh(void)
{
    return(mesh_type);
}

/*--------------------------------------------------------
    function: Gravity
    purpose:	  performs all gravity calculation based on
			   simplified physics
  -------------------------------------------------------*/
void Model::Gravity(void)
{
    if ( stopped )
    	return;

	if ( ( pos.y <= vert_speed+size.x)&&( vert_speed < 0.05)&&( vert_speed > -0.05) )
	{
	  isFalling = false;
	  isBouncing = false;
	  timer = 0 ;
	  vert_speed = 0;
	  stopped = true;
	  return;
	}
	
    if (isFalling)
    {
        vert_speed += -0.5*9.81*timer*timer;
	
        if (pos.y + vert_speed> ground+size.x)
        {
            pos.y += vert_speed ;
            height -= vert_speed ;
		    return;
        }
        else
        {
	      pos.y = ground + size.x;
	      vert_speed *= 0.75 ;
	      timer = 0;
	      isFalling = false;
	      isBouncing = true;
	      return;
        }
     }   
	
    if (isBouncing)
    {
        vert_speed += -0.5*9.81*timer*timer;
	
	    if ( pos.y < ground + height*elasticity)
	    {
            pos.y -= vert_speed ;
		    return;
	    }
	    else
	    {
		    vert_speed *= 0.75 ;
		    timer = 0;
		    height = 0;
		    isFalling = true;
		    isBouncing = false;
		    return;
	    }
    }
}    
