/*---------------------------------------------------------------------------

															      Model v1.0
  Eneko Bilbao 1999
---------------------------------------------------------------------------*/

/* 
    implements all 3d properties of an object.
    including dynamics
*/

/* STATUS
    -   transformations are all working.
    -		gravity and bouncing works.
    -		collision in a box works.    
    -		deceleration works
    -		gravity support remove since the game doesn't need it.
*/

/* TODO
    -		extended material support.
    -		add more dynamic effects.
    -		add timing support
    -   add threading for the run function
*/    

/* BUGS
    -		gravity not realistic enough
    -		deceleration depends on the framerate
*/
 
#include "vertex.h"

#ifndef _MODEL_H_
#define _MODEL_H_

#define SOLID_SPHERE 	0
#define SOLID_CUBE  	 1
#define SOLID_CONE  	 2
#define CUSTOM_MESH  	3


class Model
{
	private:
            vertex	pos;             // space coordinates
            vertex	rot;             // rotations
            vertex	spd;             // motions
            vertex	rspd;            // rotation speed
            vertex	piv;             // pivot
            vertex size;
            
            vertex	top_left;        // bounds for collision detection
            vertex btm_right;		 				// bounds for collision detection
                
			float			color[4];
		    int				mesh_type;				// 3D aspect
            
            // dynamics
		    float				weight;
			float			vert_speed;
		    float				elasticity;
		    float   				height;
		    float				friction;
	                    
		    float       timer;
		    float       time_slice;
	    
		    bool        isFalling;
		    bool        isBouncing;
		    bool				onGround;
		    bool				stopped;
			float       ground;
		    bool				bound_check;
            
	public:
		Model()
		{
		  pos.x = pos.y = pos.z = 0;
		  rot.x = rot.y = rot.z = 0;
		  rspd.x = rspd.y = rspd.z = 0;
		  piv.x = piv.y = piv.z = 0;
		  mesh_type = -1;
		  spd.x = spd.y = spd.z = 0;
		  
		  // default startup values
		  timer = 0;
		  time_slice = 0.01;
		  elasticity = 0.5;
		  weight = 0.1 ;
		  height = 0;
		  
		  isFalling = false;
		  isBouncing = false;
		  onGround = false;
		  stopped = true;
		  
		  friction = 0.02;
		  bound_check = true;
		 }
			
		~Model()
		{ }
			
		void	setPosition(float, float, float);
		void	setPosition(vertex&);
            
		void	setRotation(float, float, float);
		void	setRotation(vertex&);
            
		void	setMotion(float, float, float);
		void	setMotion(vertex&);
            
		void	setRotationSpeed(float, float, float);
		void	setRotationSpeed(vertex&);

		void	setSpeed(float, float, float);
		void	setSpeed(vertex&);

		void setPivot(float,float,float);
		void setPivot(vertex&);
            
		void resetTimer(void)
		{   timer = 0 ; }
            
		void	setTimeSampling(float f)
		{   time_slice = f ; }
            
		void	Move(float=0, float=0, float=0);
		void	Rotate(float=0,float=0,float=0);
		void	Gravity(void);

		// threading would be nice for this function
		inline void    Run(void)
		{
		  /* no gravity in this game, on test less :)
		  if (isFalling)
		    timer += time_slice ;
		  else
		    timer -= time_slice ;
		  */	
		  
		  Move();
		  Rotate();
		  
		  /* useless no gravity in this game */
		  //		  Gravity();
		}
            
		inline vertex& getPosition(void)
		{ return(pos); }
            
		inline vertex& getRotation(void)
		{ return(rot); }
            
		inline vertex& getSpeed(void)
		{ return(spd); }
            
		inline vertex& getRotationSpeed(void)
		{ return(rspd); }
            
		inline vertex& getPivot(void)
		{ return(piv); }
            
		void	setMesh(int);
		int 	getMesh(void);
            
		void	setSize(float a,float b, float c)
		{   size.x = a ;  size.y=b; size.z =c ;}
            
		vertex& getSize(void)
		{   return (size) ;}
            
		void	setGround(float g)
		{   ground = g ; }
            
		float 	getGround(void)
		{   return (ground) ;}
            
		void	setElasticity(float e)
		{   elasticity = e ; }
            
		float 	getElasticity(void)
		{   return (elasticity) ;}
		
		inline void	setColor(float r, float v, float b, float a)
		{
		  color[0] = r;
		  color[1] = v;
		  color[2] = b;
		  color[3] = a;
		}
		
		inline float	*getColor(void)
		{	return(color);	}
		
		void	letFall(void)
		{
		  if (onGround)
		    return;
				
		  stopped = false;
		  isFalling = true;
		  isBouncing =false;
		}
	
		void	stopFalling(void)
		{
		  stopped = true;
		}
    
	    void setUpperLimit( float a, float b, float c)
	    {
	      top_left.x = a;
	      top_left.y = b;
	      top_left.z = c;
	    }
    
	    void setDownLimit( float a, float b, float c)
	    {
	      btm_right.x = a;
	      btm_right.y = b;
	      btm_right.z = c;
	    }
		
	    void setFriction(float a)
	    { friction = a ;}
	    
	    void setBoundCheck(bool a)
	    { bound_check = a ; }
		    
};

#endif /*----------------------------------------*/
