#include "texture.h"

/*--------------------------------------------------------
    function: Init
    parameters: a GIF bitmap file
    purpose:	  a post init version of the constructor.
  -------------------------------------------------------*/
void Texture::Init(char* filename)
{
    imageLoader pic;
                
    tex_ref = 0;
	
    pic.loadGIF(filename);
    pic.setColorAlignement(RGBA);
    
	width = (GLuint)pic.getWidth();
    height = (GLuint)pic.getHeight();
                
    pixmap = new GLubyte[width*height*4];
    
	pic.copyImageToCharmap(pixmap, RGBA);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    
    glGenTextures(1, &tex_ref);
    glBindTexture(GL_TEXTURE_2D, tex_ref);
    
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

                
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 
                                0, GL_RGBA, GL_UNSIGNED_BYTE, pixmap);
}
            
/*--------------------------------------------------------
    function: getRef
    purpose:  used when you want to apply the texture on an
			   object
    returns:  the openGL reference of the texture
  -------------------------------------------------------*/
GLuint Texture::getRef(void)
{
    return(tex_ref);
}

/*--------------------------------------------------------
    function: getPixmap
    returns:  a pointer on the pixmap
  -------------------------------------------------------*/
GLubyte *Texture::getPixmap(void)
{
    return(pixmap);
}

/*--------------------------------------------------------
    function: getWidth
    returns:  the width of the image IN PIXELS
  -------------------------------------------------------*/
GLuint Texture::getWidth()
{
    return(width);
}

/*--------------------------------------------------------
    function: getHeight
    returns:  the height of the image IN PIXELS
  -------------------------------------------------------*/
GLuint Texture::getHeight()
{
    return(height);
}

/*--------------------------------------------------------
    function: Reset
    purpose:	 resets the object to be re-used
  -------------------------------------------------------*/
void Texture::Reset(void)
{
    width = height = 0;
                
    if (pixmap!=0)
	    delete[] pixmap;
}

